#____________________________________________________________________________________
#
#    PersistentThreads (see README.txt for more information)
#    Copyright (C) 2002 Christoph Becker-Freyseng
#
#    This library is free software; you can redistribute it and/or
#    modify it under the terms of the GNU Lesser General Public
#    License as published by the Free Software Foundation; either
#    version 2.1 of the License, or (at your option) any later version.
#
#    This library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public
#    License along with this library; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
#    christoph@mmc-startup.com
#    http://www.beyond-thoughts.com
#    http://www.mmc-startup.com
#____________________________________________________________________________________
#



__doc__ = """PTTools: some useful classes, methods, ... that are used with PThreads"""
__version__ = '0.2.1'  

from PConstants import ppHandler_permission, pThreadTool_permission

import Globals
from App.Common import aq_base






def _getAppObj(zObj):
    if hasattr(zObj, 'getPhysicalRoot'):
        app= zObj.getPhysicalRoot()
    else:	# Try really hard to get it ....
        app= None
        objR= zObj
        while objR:
            if hasattr(aq_base(objR), 'isTopLevelPrincipiaApplicationObject'):
                if objR.isTopLevelPrincipiaApplicationObject:   # Maybe use objR.aq_base to not acquire isTopLevelPrincipiaApplicationObject
                    app= objR
                    break
            if hasattr(objR, 'aq_parent'):
                objR= objR.aq_parent
            else:
                break
    return app

def resolvePath(zObj, path):
    """ Try to resolve this path via Acquisistion and return the object.
	If the object can't be found return None (this is more useful than an exception).
	If the path has a leading / it is treated "absolute" -> starting to resolve
	from the app object -- otherwise start from zObj. """
    pathTuple= filter(None, path.split('/'))	# make a tuple of the path removing empty items
    if not pathTuple:	# when there are no items we can't find something
	return None
    if path[0]=='/':
	initObj= _getAppObj(zObj)
    else:
	initObj= zObj
    try:
	return reduce(lambda objR,pItem: objR[pItem], pathTuple, initObj)	# iterate through pathTuple (which can't be empty) always getting the next objR as attribute of the old one. Acquisition should work here automagically -- if not use objR[pItem].__of__(objR)
    except:
	return None








def initPermissions():
    import Products
    Products.__ac_permissions__=Products.__ac_permissions__ + (
	    (ppHandler_permission, (), ('Manager',) ),
	    (pThreadTool_permission, (), ('Manager',) ),
	)



def checkPermission(permiP, obj2AccP, REQUEST): # returns None if permission OK otherwise returns errorMsg
    if (REQUEST is None):
	return 'No REQUEST -> No Permission\n'
    else:
	userD= REQUEST.get('AUTHENTICATED_USER', None)
    if (userD is None):
	return 'No User -> No Permission\n'
    else:
	if not userD.has_permission(permiP, obj2AccP):
	    return 'User "' + str(userD) + '" has no Permission to "' + str(permiP) + '".\n'
    return None





_commonTabPageDTML = Globals.DTMLFile('commonTabPage', globals())

def wrapHTML(objP, titleP, htmlP, REQUEST=None):
    """ returns your HTML-code, but wrapped within a HTML, so that there are management_tabs ..."""
    dtmlD= _commonTabPageDTML.__of__(objP)
    return dtmlD(objP, REQUEST, page_title=titleP, page_str=htmlP)










_ui_PPs= {}

def registerUI_PP(classP, nameP):
    """ Register a PersistentProcessor with a UI, to you can add it with PThreadTool """
    if not getattr(classP, 'isA_UI_PProcessor', None):
	return 0
    if _ui_PPs.has_key(nameP):
	return 0
    _ui_PPs[nameP]= classP
    return 1

def getUI_PP(nameP):
    """ get a UI_PP via name """
    return _ui_PPs.get(nameP, None)

def registeredUI_PPs():
    """ return a list of the name of the registered UI_PPs """
    return _ui_PPs.keys()





def getNumberOfThreads():
    import ZServer.PubCore
    return ZServer.PubCore.getNumberOfThreads()

def getNumberOfThreadsOMG():	# Whaaaaaa in ZServer.PubCore.__init__.py they implemented set... but no get... WHHHHHAAAAAA
    try:
	import ZServer.PubCore
	return ZServer.PubCore._n
    except:
	return 0	# ??????????

def getNumberOfThreadsINIT():
    import ZServer.PubCore
    ZServer.PubCore.getNumberOfThreads= getNumberOfThreadsOMG

getNumberOfThreadsINIT()
















